clear
clc
rng(12345)
tic
Tt=206;


N=20000;
tol = .01;% ABC tolerance
rep = 1;
m=1; % Poisson arrivals */
crps_point = 2;


AR = 1; % AR term in INAR
MA = 0; % MA term in INAR

alpha_val =.40000; % persistence parameter
beta_val = .000;
lambda_val = 2;

params = [lambda_val,alpha_val,beta_val]';

%% Fitting Standard INARMA %%
[y_all,y_all1] = simulation_file_INARMA(params,Tt,m,AR,MA);

for l=1:1
ydata=y_all(1:99+l); % Data using to fit the model
T=length(ydata); % Data size
y_tp1 = y_all(99+l+1) %Value to be predicted

% Poisson,lambda,alpha
prior_alpha = rand(N,1);
prior_beta = zeros(N,1);
prior_lambda =  10.*rand(N,1);%gamrnd(2,2,[N,1]);
params = [prior_lambda,prior_alpha,prior_beta];

% Standard INAR %
tic
y_sim=ABC_sim_file_INARMA(params,N,T,m); % Generating simulated data

mu_y = mean(ydata);
yT1 = [0;ydata(1:(T-1))];
yT2 = [0;0;ydata(1:(T-2))];
yT3 = [0;0;0;ydata(1:(T-3))];

X = [ones(T,1) yT1 yT2];
Ar_coeffs = X\(ydata);
sig_y = var(ydata);
data_C = [mu_y; sig_y; Ar_coeffs];

% Simulated autoregressions 
yT1S = [zeros(1,N);y_sim(1:(T-1),:)];
yT2S = [zeros(1,N);zeros(1,N);y_sim(1:(T-2),:)];
yT3S = [zeros(1,N);zeros(1,N);zeros(1,N);y_sim(1:(T-3),:)];

% Simulated Summary statistics
rho_1 = mean((ydata-mu_y).*(yT1-mu_y))./sqrt(var(yT1-mu_y)*var(ydata-mu_y));
rho_1S = mean((y_sim-ones(T,1)*mean(y_sim)).*(yT1S-ones(T,1)*mean(y_sim)))./sqrt(var(yT1S-ones(T,1)*mean(y_sim)).*var(y_sim-ones(T,1)*mean(y_sim)));
rho_2 = mean((ydata-mu_y).*(yT2-mu_y))./sqrt(var(yT2-mu_y)*var(ydata-mu_y));
rho_2S = mean((y_sim-ones(T,1)*mean(y_sim)).*(yT2S-ones(T,1)*mean(y_sim)))./sqrt(var(yT2S-ones(T,1)*mean(y_sim)).*var(y_sim-ones(T,1)*mean(y_sim)));
rho_3 = mean((ydata-mu_y).*(yT3-mu_y))./sqrt(var(yT3-mu_y)*var(ydata-mu_y));
rho_3S = mean((y_sim-ones(T,1)*mean(y_sim)).*(yT3S-ones(T,1)*mean(y_sim)))./sqrt(var(yT3S-ones(T,1)*mean(y_sim)).*var(y_sim-ones(T,1)*mean(y_sim)));

% Using observed data to form the weighting matrix
temp_0 = (ydata-mu_y);
temp_1 = ((ydata-mu_y).*(yT1-mu_y))./sqrt(var(yT1-mu_y)*var(ydata-mu_y));
temp_2 = ((ydata-mu_y).*(yT2-mu_y))./sqrt(var(yT2-mu_y)*var(ydata-mu_y));
temp_3 = ((ydata-mu_y).*(yT3-mu_y))./sqrt(var(yT3-mu_y)*var(ydata-mu_y));
Temp = [temp_0 temp_1 temp_2 temp_3];
% Weighting matrix
W_mat=inv(Temp'*Temp./T);
% Simulated moments
MoM = [mean(y_sim,1)-mu_y;rho_1S-rho_1;rho_2S-rho_2;rho_3S-rho_3]; 
% ABC loop 
Metric = zeros(1,N);
for i=1:N
  Metric(:,i) =  sqrt(  (MoM(:,i))'*W_mat*(MoM(:,i))); 
end
% Setting the lowest value of epsilon to keep
Keep = quantile(Metric,.01);
% Collecting the accepted ABC draws. 
ABC_alpha = prior_alpha(Metric<Keep);
ABC_beta = prior_beta(Metric<Keep);
ABC_lam =prior_lambda(Metric<Keep);

% Time stamp
time_abc(l,1) = toc;

%% Kernel densities of accepted draws
[f_alpha,x_alpha] = ksdensity(ABC_alpha,linspace(0,1,100));
[f_lam,x_lam] = ksdensity(ABC_lam,linspace(0,10,100)); 
[f_beta,x_beta] = ksdensity(ABC_beta,linspace(min(ABC_beta),max(ABC_beta),500));

[f_alpha1,x_alpha1] = ksdensity(ABC_alpha, ABC_alpha);
[f_lam1,x_lam1] = ksdensity(ABC_lam, ABC_lam); 

%% Setting up the predictive 
Pred_vals = 0:1:max(ydata);
prob_preds = zeros(length(Pred_vals),1);

for i=1:length(Pred_vals)
     prob_preds(i,1) = mean(poiss_pred(Pred_vals(i), ydata, ABC_lam, ABC_alpha,1));
     preds = poiss_pred(Pred_vals(i), ydata, x_lam1, x_alpha1,1);
end

Pred_T1 = prob_preds./sum(prob_preds);

% Comment the below line to generate the bar plots: 
% bar(Pred_vals, Pred_T1)

val=(Pred_vals==y_tp1);

%% Generating true predictive to calculate the exact version of the expected scoring rules 
true_preds=zeros(length(Pred_vals),1);
for i=1:length(Pred_vals)
    prob_preds(i,1) = mean(poiss_pred(Pred_vals(i), ydata, lambda_val, alpha_val,1));
      true_preds(i,1) = poiss_pred(Pred_vals(i), ydata, lambda_val, alpha_val,1);
end
True_Pred = true_preds;
TRUE_Pred(l,1) = true_preds(val);
%% Exact Predictive 
 
tic
y_t_1 = [0;ydata(1:end-1)]; % Lagged outcomes. 
M_t = bsxfun(@min,ydata, y_t_1);
Nn = 2000; % Number of grid points 

alpha_temp=linspace(0.01,.9,Nn); % Grid for exact posterior on alpha
lambda_temp=linspace(0.01,10,Nn); % Grid for exact posterior on lambda

[post_L,post_A] = jpost(T,ydata,M_t,alpha_temp,lambda_temp,Nn); % Generating joint exact posterior


% Comment out to plot the posteriors 
% subplot(1,3,2)
% plot(lambda_temp,post_L,'black',x_lam,f_lam,'red')
% subplot(1,3,1)
% plot(alpha_temp,post_A,'black',x_alpha,f_alpha,'red')


%% Need to re-weight the posterior draws appropriately to generate the predictive. Conversely, could do numerically.
%% rejection sampler for alpha_exact
c=max(post_A)+.1;
x = .1+.8.*rand(N,1);
lambda = rand(N,1);
LHS = lambda.*c.*x;
f_xa = interp1(alpha_temp,post_A,x);
vals_a = (f_xa>LHS);
temp_x = x(vals_a);

alpha_sam = randsample(temp_x,length(x_lam1),'true');

%% rejection sampler for lambda_exact
c=max(post_L)+.1;
x = .5+5.5.*rand(N,1);
lambda = rand(N,1);
LHS = lambda.*c.*x;
f_xl = interp1(lambda_temp,post_L,x);
vals_l = (f_xl>LHS);
temp_x = x(vals_l);

lambda_sam = randsample(temp_x,length(x_lam1),'true');
post_l_vals = interp1(lambda_temp,post_L,lambda_sam);
time_exact(l,1) = toc;

%% Constructing exact predictive

prob_preds1=zeros(length(Pred_vals),1);

for i=1:length(Pred_vals)
      preds = poiss_pred(Pred_vals(i), ydata, lambda_sam, alpha_sam,1);
      prob_preds1(i,1) = mean(preds);
end

preds_exact = prob_preds1;

% Uncomment to plot
% plot(Pred_vals,prob_preds1)
% subplot(1,2,2)
% bar(Pred_vals,preds_exact)


% Scores for ABC and Exact methods across the holdout periods
TV =  .5*sum(abs(Pred_T1-preds_exact));
score_log_ABC(l,1) = sum(True_Pred.*log(Pred_T1+.001));
score_qs_ABC(l,1) = sum(True_Pred.*(2.*Pred_T1-sum(Pred_T1.^2)));
score_log_Exact(l,1) = sum(True_Pred.*log(preds_exact+.001));
score_qs_Exact(l,1) = sum(True_Pred.*(2.*preds_exact-sum(preds_exact.^2)));
l
end